/**
 * Copyright (c) 2021, WSO2 Inc. (http://www.wso2.com). All Rights Reserved.
 *
 * This software is the property of WSO2 Inc. and its suppliers, if any.
 * Dissemination of any information or reproduction of any material contained
 * herein in any form is strictly forbidden, unless permitted by WSO2 expressly.
 * You may not alter or remove any copyright or other notice from copies of this content."
 */

import { AlertLevels, TestableComponentInterface } from "@wso2is/core/models";
import { addAlert } from "@wso2is/core/store";
import get from "lodash-es/get";
import React, { FunctionComponent, ReactElement, SyntheticEvent, useEffect, useState } from "react";
import { useTranslation } from "react-i18next";
import { useDispatch } from "react-redux";
import { Checkbox, CheckboxProps } from "semantic-ui-react";
import { SettingsSection } from "./settings-section";
import { AppConstants, history } from "../../../../features/core";
import { updateGovernanceConnector } from "../../../../features/server-configurations/api";
import { getGovernanceConnectorIllustrations } from "../../../../features/server-configurations/configs";
import { GovernanceConnectorInterface } from "../../../../features/server-configurations/models";
import { GovernanceConnectorUtils } from "../../../../features/server-configurations/utils";

/**
 * Prop types for the edit connector component.
 */
interface EditConnectorProps extends TestableComponentInterface {
    connector: GovernanceConnectorInterface;
    connectorToggleName: string;
    categoryID: string;
    onUpdate: () => void;
}

/**
 * Edit connector component.
 *
 * @param {EditConnectorProps} props - Props injected to the connector component.
 *
 * @return {React.ReactElement}
 */
export const EditConnector: FunctionComponent<EditConnectorProps> = (
    props: EditConnectorProps
): ReactElement => {
    const {
        connector,
        onUpdate,
        categoryID,
        connectorToggleName,
        ["data-testid"]: testId
    } = props;

    const dispatch = useDispatch();

    const { t } = useTranslation();

    const [ connectorIllustration, setConnectorIllustration ] = useState<string>(undefined);
    const [ showActionBar, setShowActionBar ] = useState<boolean>(false);
    const [ enableOption, setEnableOption ] = useState<boolean>(false);

    /**
     * Initial connector status.
     */
    useEffect(() => {
        if (connector) {
            const enableProperty = connector.properties.find((property) => property.name === connectorToggleName);

            setEnableOption(enableProperty?.value === "true");
        }
    }, [ connector ]);

    /**
     * Show/Hide action bar.
     */
    useEffect(() => {
        setShowActionBar(enableOption);
    }, [ enableOption ]);

    /**
     * This is called when the enable toggle changes.
     *
     * @param {SyntheticEvent} e Event object
     * @param {CheckboxProps} data The data object.
     */
    const handleToggle = (e: SyntheticEvent, data: CheckboxProps): void => {
        setEnableOption(data.checked);

        const updateData = {
            operation: "UPDATE",
            properties: []
        };

        updateData.properties.push({
            name: GovernanceConnectorUtils.decodeConnectorPropertyName(connectorToggleName),
            value: data.checked.toString()
        });

        updateGovernanceConnector(updateData, connector.categoryId, connector.id)
            .then(() => {
                dispatch(
                    addAlert({
                        description: t(
                            "console:manage.features.governanceConnectors.notifications." +
                            "updateConnector.success.description",
                            { name: connector.friendlyName }
                        ),
                        level: AlertLevels.SUCCESS,
                        message: t(
                            "console:manage.features.governanceConnectors.notifications." +
                            "updateConnector.success.message"
                        )
                    })
                );
            })
            .catch((error) => {
                if (error.response && error.response.data && error.response.data.detail) {
                    dispatch(
                        addAlert({
                            description: t(
                                "console:manage.features.governanceConnectors.notifications." +
                                "updateConnector.error.description",
                                { description: error.response.data.description }
                            ),
                            level: AlertLevels.ERROR,
                            message: t(
                                "console:manage.features.governanceConnectors.notifications." +
                                "updateConnector.error.message"
                            )
                        })
                    );
                } else {
                    // Generic error message
                    dispatch(
                        addAlert({
                            description: t(
                                "console:manage.features.governanceConnectors.notifications." +
                                "updateConnector.genericError.description"
                            ),
                            level: AlertLevels.ERROR,
                            message: t(
                                "console:manage.features.governanceConnectors.notifications." +
                                "updateConnector.genericError.message"
                            )
                        })
                    );
                }
            });
    };

    /**
     * Set the connector illustration.
     */
    useEffect(() => {
        if (!connector || !connector.id) {
            return;
        }

        const illustration: string = get(getGovernanceConnectorIllustrations(), connector.id,
            getGovernanceConnectorIllustrations()?.default);

        setConnectorIllustration(illustration);
    }, [connector, getGovernanceConnectorIllustrations]);

    /**
     * Handle connector advance setting selection.
     */
    const handleSelection = () => {
        history.push(AppConstants.getPaths()
            .get("GOVERNANCE_CONNECTOR_EDIT")
            .replace(":categoryId", categoryID)
            .replace(":connectorId", connector?.id));
    };

    /**
     * This renders the enable toggle.
     */
    const connectorToggle = (): ReactElement => {
        return (
            <Checkbox
                label={ "Enable " + connector.friendlyName }
                toggle
                onChange={ handleToggle }
                checked={ enableOption }
                data-testId={ `${testId}-enable-toggle` }
            />
        );
    };

    return (
        <SettingsSection
            data-testid={ `${testId}-settings-section` }
            description={ t("console:manage.features.governanceConnectors.connectorSubHeading", {
                name: connector?.friendlyName
            }) }
            header={ connector?.friendlyName }
            icon={ connectorIllustration }
            iconMini={ connectorIllustration }
            iconSize="auto"
            iconStyle="colored"
            iconFloated="right"
            onPrimaryActionClick={ handleSelection }
            primaryAction={ "Advance Settings" }
            showActionBar={ showActionBar }
        >
            <div className={ showActionBar ? "ml-5 mb-3" : "ml-5 mb-5" }>
                { connectorToggle() }
            </div>
        </SettingsSection>
    );
};

/**
 * Default props for the component.
 */
EditConnector.defaultProps = {
    "data-testid": "dynamic-governance-connector"
};
